import os
import pathlib

from flask import current_app
from jinja2 import Environment, PackageLoader
from sqlalchemy import Column

from flask_sample_test import SampleTest
from flask_sample_test.fields import TestColumn

try:
    from flask_script import Manager
except ImportError:
    Manager = None


if Manager is not None:
    SampleTestCommand = Manager(usage='Perform sample test')
else:
    class FakeCommand(object):
        def option(self, *args, **kwargs):
            def decorator(f):
                return f
            return decorator

    SampleTestCommand = FakeCommand()


@SampleTestCommand.option('-m', '--model', dest='model_name')
@SampleTestCommand.option('-q', '--quantity', dest='quantity', type=int)
def create_sample_data(model_name, quantity):
    flask_sample_test: SampleTest = current_app.extensions['flask_sample_test']
    test_data_path = current_app.config['TEST_DATA_PATH']

    env = Environment(loader=PackageLoader(pathlib.Path(__file__).resolve().parent.name))
    template = env.get_template('test_data.txt')

    model = flask_sample_test.models.get(model_name)
    if not model:
        raise Exception(f"unknown model, model_name: {model_name}")
    columns = model.__table__.columns
    test_file = os.path.join(os.getcwd(), test_data_path, f"{model_name}.py")
    with open(test_file, 'a+') as f:
        is_empty_file = True if not os.path.getsize(test_file) else False
        mock_data_list = []
        for i in range(quantity):
            random_data = {}
            for column in columns:
                column: Column
                if column.primary_key:
                    continue
                test_column = TestColumn.init_field(column)
                random_data[column.name] = test_column.random_value()
            mock_data_list.append(random_data)
        context = {
            "mock_data": mock_data_list,
            "import_model": f"from {model.__module__} import {model.__name__}",
            "model_name": model.__name__,
            "is_empty_file": is_empty_file,
        }
        content = template.render(**context)
        f.write(content)



import json
from typing import List, Union


class SampleTestJSONEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, SimpleDataField):
            return str(o)
        return super().default(o)


class SampleTest(object):
    def __init__(self, app=None, db=None):
        self.app = app
        self.db = db
        self.models = {}
        self.config = {}
        if app is not None and db is not None:
            self.init_app(app, db)

    def init_app(self, app, db=None):
        self.db = db or self.db
        if not hasattr(app, 'extensions'):
            app.extensions = {}
        self.models = {mapper.class_.__tablename__: mapper.class_ for mapper in db.Model.registry.mappers}
        app.extensions['flask_sample_test'] = self
        app.config.setdefault('TEST_DATA_PATH', 'sample_data')

    def create_env(self, sample_data_list):
        return SampleEnvironment(sample_data_list=sample_data_list, bind=self)

    def create_sample_data(self, model, data):
        return SampleData(model=model, data=data, bind=self)


class SampleData:
    def __init__(self, model, data, bind: SampleTest):
        self._model: callable = model
        self._data: dict = data
        self._instance = None
        self._bind = bind

    def create_instance(self):
        if self._instance:
            return self._instance.id
        self._instance = self._model()

        columns = self._model.__table__.columns
        for column in columns:
            column_name = column.get('name')
            column_nullable = column.get('nullable')
            column_type = column.get('type')
            column_default = column.get('default')
            if column_name == 'id':
                continue
            if column_name in self._data:
                data = self._data.get(column_name)
                if isinstance(column_type, self._db.Integer) and \
                        isinstance(data, SampleData):
                    fk = data.create_instance()
                    setattr(self._instance, column_name, fk)
                elif isinstance(data, (SimpleDataField, ComplexDataField)):
                    setattr(self._instance, column_name, str(data))
                else:
                    setattr(self._instance, column_name, data)
            assert column_name in self._data or column_nullable is True or column_default is not None, \
                f'{self._model.__name__.lower()} 的非空字段 {column_name} 不能为空'

        self._db.session.add(self._instance)
        self._db.session.flush()

        return self._instance.id

    def relation_model(self):
        result = [self]
        for key, value in self.data.items():
            if isinstance(value, SampleData):
                result.extend(value.relation_model())
        return result

    def init_instance(self):
        self._instance = None

    def load_instance(self):
        self._instance = self._db.session.merge(self._instance)

        return self._instance

    def __getattr__(self, item):
        self._instance = self.load_instance()
        return getattr(self._instance, item, None)

    def __setattr__(self, key, value):
        if key in ('_model', '_data', '_instance', '_db', '_inspector'):
            super(SampleData, self).__setattr__(key, value)
            return

        self._instance = self.load_instance()
        setattr(self._instance, key, value)


class SimpleDataField:
    def __init__(self, sample_data: SampleData, field: str):
        """
        :param sample_data: 样例数据
        :param field: 要取的字段
        """
        self._sample_data = sample_data
        self._field = field

    def __str__(self):

        self._sample_data.create_instance()
        return getattr(self._sample_data, self._field)

    def init_instance(self):
        self._sample_data.init_instance()


class ComplexDataField:
    def __init__(self, data: Union[dict, list, tuple, int, float, bool, str]):
        """
        :param data: 数据,
        """
        self.data = data
        self.relation: List[Union[SampleData, SimpleDataField]] = []

    def __str__(self):
        result = json.dumps(self.data, cls=SampleTestJSONEncoder)
        return json.loads(result)

    def _relation_simple_data(self, obj):
        if isinstance(obj, (str, float, int, bool)):
            return

        if isinstance(obj, SimpleDataField):
            self.relation.append(obj)

        if isinstance(obj, dict):
            for key, value in obj.items():
                self._relation_simple_data(key)
                self._relation_simple_data(value)
        if isinstance(obj, (list, tuple)):
            for item in obj:
                self._relation_simple_data(item)

    def init_instance(self):
        for item in self.relation:
            item.init_instance()


class SampleEnvironment(object):

    def __init__(self, sample_data_list, bind: SampleTest):
        self.bind = bind
        self.sample_data_list = sample_data_list

    def __enter__(self):
        for ins in self.sample_data_list:
            ins.create_instance()
        self.bind.db.session.commit()
        self.bind.db.session.close()

    def __exit__(self, exc_type, exc_val, exc_tb):
        for ins in self.sample_data_list:
            ins.init_instance()
        for table in reversed(self.bind.db.metadata.sorted_tables):
            self.bind.db.session.query(table).delete()
            self.bind.db.session.commit()
            self.bind.db.session.close()



from setuptools import setup, find_packages
import pathlib


here = pathlib.Path(__file__).parent.resolve()

long_description = (here / 'README.md').read_text(encoding='utf-8')

setup(
    name='flask-sample-test',  # Required
    version='0.0.4',  # Required
    description='A sample test for project',  # Optional
    long_description=long_description,  # Optional
    long_description_content_type='text/markdown',  # Optional (see note above)
    url='https://github.com/December1208/flask_sample_test',  # Optional
    author='JiaLe.tan',  # Optional
    author_email='jiale.gfi@outlook.com',  # Optional
    keywords='flask, test, sqlalchemy',  # Optional
    packages=find_packages(),  # Required
    install_requires=['flask_sqlalchemy >= 2.0'],  # Optional
    python_requires='>=3.7',
    classifiers=[
      "Programming Language :: Python :: 3",
      "License :: OSI Approved :: MIT License",
      "Operating System :: OS Independent",
    ],
)




import datetime
import json
import random
import string

from sqlalchemy import Column, Integer, INT, String, Text, JSON, ARRAY, Date, DateTime, TIMESTAMP, Boolean


class TestColumn:

    @classmethod
    def init_field(cls, column: Column):
        if isinstance(column.type, (Integer, INT)):
            return IntField(column)
        elif isinstance(column.type, (String, Text)):
            return StringField(column)
        elif isinstance(column.type, JSON):
            return JsonField(column)
        elif isinstance(column.type, ARRAY):
            return ArrayField(column)
        elif isinstance(column.type, Date):
            return DateField(column)
        elif isinstance(column.type, DateTime):
            return DateTimeField(column)
        elif isinstance(column.type, TIMESTAMP):
            return TimestampField(column)
        elif isinstance(column.type, Boolean):
            return BooleanField(column)


class Field:

    def __init__(self, column: Column):
        self._column = column

    def random_value(self):
        raise NotImplementedError

    def _random_int(self, max_num=100):
        return random.randint(0, max_num)

    def _random_string(self, max_length):
        length = random.randint(0, max_length)
        return ''.join(random.choices(string.digits + string.ascii_letters, k=length))

    def _random_json(self):
        length = random.randint(0, 10)
        result = {
            ''.join(random.choices(string.digits + string.ascii_letters, k=8)): i
            for i in range(length)
        }

        return result

    def _random_text(self):
        return random.sample(string.digits + string.ascii_letters, 100)

    def _random_list(self):
        length = random.randint(0, 10)
        return [random.sample(string.digits + string.ascii_letters, i) for i in range(length)]

    def _random_date(self):
        return datetime.date.today()

    def _random_datetime(self):
        return datetime.datetime.utcnow()

    def _random_array(self, child_type: "Field"):
        length = random.randint(0, 10)
        return [child_type.random_value() for i in range(length)]

    def _random_timestamp(self):
        return datetime.datetime.now().timestamp()

    def _random_boolean(self):
        return bool(random.randint(0, 2))


class IntField(Field):
    def __init__(self, column):
        super(IntField, self).__init__(column)

    def random_value(self):
        return self._random_int()


class BooleanField(Field):
    def __init__(self, column):
        super(BooleanField, self).__init__(column)

    def random_value(self):
        return self._random_boolean()


class StringField(Field):
    def __init__(self, column):
        super(StringField, self).__init__(column)
        self.max_length = 100 if type(column.type) == Text else column.type.length
        self.default = column.default or column.server_default

    def random_value(self):
        if self.default == "{}":
            return json.dumps(self._random_json())
        elif self.default == "[]":
            return json.dumps(self._random_list())
        return self._random_string(self.max_length)


class JsonField(Field):
    def __init__(self, column):
        super(JsonField, self).__init__(column)

    def random_value(self):
        return self._random_json()


class ArrayField(Field):
    def __init__(self, column):
        super(ArrayField, self).__init__(column)

        self._item_type = column.type.item_type
        self._item_type_ins = TestColumn.init_field(Column(self._item_type))

    def random_value(self):
        return self._random_array(self._item_type_ins)


class DateField(Field):
    def __init__(self, column):
        super(DateField, self).__init__(column)

    def random_value(self):
        return self._random_date()


class DateTimeField(Field):
    def __init__(self, column):
        super(DateTimeField, self).__init__(column)

    def random_value(self):
        return self._random_datetime()


class TimestampField(Field):
    def __init__(self, column):
        super(TimestampField, self).__init__(column)

    def random_value(self):
        return self._random_timestamp()
